import datetime

from django.conf import settings
from django.db import models

from django.contrib.auth.models import User
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType


CONTEXT_STRING_OVERRIDES = getattr(settings, "POTAMOS_CONTEXT_STRING_OVERRIDES", {})


class Event(models.Model):
    
    context = models.CharField(max_length=20)
    
    user = models.ForeignKey(User, null=True)
    target_content_type = models.ForeignKey(ContentType, null=True)
    target_object_id = models.IntegerField(null=True)
    target_object = generic.GenericForeignKey("target_content_type", "target_object_id")
    user2 = models.ForeignKey(User, null=True, related_name="events_referenced")
    observer = models.ForeignKey(User, null=True, related_name="events_observed")
    
    kind = models.CharField(max_length=20)
    text = models.TextField()
    occurred = models.DateTimeField(default=datetime.datetime.now)
    
    class Meta:
        ordering = ["-occurred"]


def issue_event(kind, contexts, observers=None):
    context_data = {}
    context_data_strings = {}
    insert_values = []
    now = datetime.datetime.now()
    
    # Format messages
    for context, context_values in contexts.iteritems():
        if len(context_values) == 2:
            value = context_values[0]
            context_data.update({
                context: value,
            })
            if context in CONTEXT_STRING_OVERRIDES:
                func = CONTEXT_STRING_OVERRIDES[context]
                value_string = func(value)
            elif hasattr(value, "potamos_string_value") and callable(value.potamos_string_value):
                value_string = value.potamos_string_value(value)
            else:
                value_string = unicode(value)
            context_data_strings.update({
                context: value_string,
            })
    
    # Build up data to insert
    for context, context_values in contexts.iteritems():
        if len(context_values) == 1:
            text = context_values[0]
        else:
            # assume len() == 2
            text = context_values[1]
        insert_kwargs = {
            "context": context,
            "kind": kind,
            "text": text % context_data_strings,
            "occurred": now
        }
        insert_kwargs.update(context_data)
        insert_values.append(insert_kwargs)
    
    # Add observers to data to insert
    if observers is not None:
        for observer in observers:
            insert_kwargs = {
                "context": "observer",
                "kind": kind,
                "text": contexts["global"][0] % context_data_strings,
                "observer": observer,
                "occurred": now
            }
            insert_values.append(insert_kwargs)
    
    # Insert data
    for insert_kwargs in insert_values:
        Event(**insert_kwargs).save()
